=begin
#TollGuru Calculator

#TollGuru Calculator API

OpenAPI spec version: 2020-07-15T09:40:00Z
Contact: eng@tollguru.com
Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 2.4.14

=end

require 'date'

module SwaggerClient
  class Truck
    # Vehicle height in feet. The provided value must be between 0 and 50. <br> If vehicle height is not specified, incorrect tolls may be calculated on New York Thruway and other toll roads where tolls are based on axles and weight.
    attr_accessor :height

    # Vehicle length in feet. The provided value must be between 0 and 300.
    attr_accessor :length

    # Vehicle weight including trailers and shipped goods, in pounds. <br> If truck weight is not specified, incorrect tolls may be calculated for trucks on Pennsylvania turnpike and other toll locations where tolls are based on weight.
    attr_accessor :limited_weight

    # List of hazardous materials in the vehicle. The names correspond to <a href=\"https://www.fmcsa.dot.gov/sites/fmcsa.dot.gov/files/docs/Nine_Classes_of_Hazardous_Materials-4-2013_508CLN.pdf\" target=\"_blank\"> HAZMAT classes 1 to 9. <br> For example, explosive is Class 1 and so on. </a>
    attr_accessor :shipped_hazardous_goods

    # Specifies number of trailers pulled by a vehicle. The provided value must be between 0 and 4.
    attr_accessor :trailers_count

    # pecifies the penalty type on violated truck restrictions.<br><code>strict</code>: route will not use links with a violated truck restriction at any circumstances<br><strong>Note</strong>: Route computation will fail and return NoRouteFound if not able to avoid the links with a violated truck restriction and the penalty type is strict.<br><code>soft</code>: route will use links with a violated truck restriction if there is no alternative to avoid them<
    attr_accessor :truck_restriction_penalty

    # Specifies the vehicle type<br><strong>Note</strong>: Relevant for restrictions that apply exclusively to tractors with semi-trailers (observed in North America). Such restrictions are taken into account only in case of the <code>truckType</code> set to <code>tractorTruck</code> and the trailers count greater than 0 (for example <code>truckType: \"tractorTruck\", trailersCount: 1</code>). The truck type is irrelevant in case of restrictions common for all types of trucks.
    attr_accessor :truck_type

    # Specifies the tunnel category to restrict certain route links. The route will pass only through tunnels of a less strict category.
    attr_accessor :tunnel_category

    # Vehicle weight per axle in pounds. <br> If truck weight is not specified, incorrect tolls may be calculated for trucks on Pennsylvania turnpike and other toll locations where tolls are based on weight.
    attr_accessor :weight_per_axle

    # Vehicle width in feet. The provided value must be between 0 and 50.
    attr_accessor :width

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'height' => :'height',
        :'length' => :'length',
        :'limited_weight' => :'limitedWeight',
        :'shipped_hazardous_goods' => :'shippedHazardousGoods',
        :'trailers_count' => :'trailersCount',
        :'truck_restriction_penalty' => :'truckRestrictionPenalty',
        :'truck_type' => :'truckType',
        :'tunnel_category' => :'tunnelCategory',
        :'weight_per_axle' => :'weightPerAxle',
        :'width' => :'width'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'height' => :'Float',
        :'length' => :'Float',
        :'limited_weight' => :'Float',
        :'shipped_hazardous_goods' => :'String',
        :'trailers_count' => :'Float',
        :'truck_restriction_penalty' => :'String',
        :'truck_type' => :'String',
        :'tunnel_category' => :'String',
        :'weight_per_axle' => :'Float',
        :'width' => :'Float'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'height')
        self.height = attributes[:'height']
      end

      if attributes.has_key?(:'length')
        self.length = attributes[:'length']
      end

      if attributes.has_key?(:'limitedWeight')
        self.limited_weight = attributes[:'limitedWeight']
      end

      if attributes.has_key?(:'shippedHazardousGoods')
        self.shipped_hazardous_goods = attributes[:'shippedHazardousGoods']
      end

      if attributes.has_key?(:'trailersCount')
        self.trailers_count = attributes[:'trailersCount']
      end

      if attributes.has_key?(:'truckRestrictionPenalty')
        self.truck_restriction_penalty = attributes[:'truckRestrictionPenalty']
      end

      if attributes.has_key?(:'truckType')
        self.truck_type = attributes[:'truckType']
      end

      if attributes.has_key?(:'tunnelCategory')
        self.tunnel_category = attributes[:'tunnelCategory']
      end

      if attributes.has_key?(:'weightPerAxle')
        self.weight_per_axle = attributes[:'weightPerAxle']
      end

      if attributes.has_key?(:'width')
        self.width = attributes[:'width']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      shipped_hazardous_goods_validator = EnumAttributeValidator.new('String', ['explosive', 'gas', 'flammable', 'combustible', 'organic', 'poison', 'radioActive', 'corrosive', 'miscellaneous'])
      return false unless shipped_hazardous_goods_validator.valid?(@shipped_hazardous_goods)
      truck_restriction_penalty_validator = EnumAttributeValidator.new('String', ['strict', 'soft'])
      return false unless truck_restriction_penalty_validator.valid?(@truck_restriction_penalty)
      truck_type_validator = EnumAttributeValidator.new('String', ['truck', 'tractorTruck'])
      return false unless truck_type_validator.valid?(@truck_type)
      tunnel_category_validator = EnumAttributeValidator.new('String', ['B', 'C', 'D', 'E'])
      return false unless tunnel_category_validator.valid?(@tunnel_category)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] shipped_hazardous_goods Object to be assigned
    def shipped_hazardous_goods=(shipped_hazardous_goods)
      validator = EnumAttributeValidator.new('String', ['explosive', 'gas', 'flammable', 'combustible', 'organic', 'poison', 'radioActive', 'corrosive', 'miscellaneous'])
      unless validator.valid?(shipped_hazardous_goods)
        fail ArgumentError, 'invalid value for "shipped_hazardous_goods", must be one of #{validator.allowable_values}.'
      end
      @shipped_hazardous_goods = shipped_hazardous_goods
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] truck_restriction_penalty Object to be assigned
    def truck_restriction_penalty=(truck_restriction_penalty)
      validator = EnumAttributeValidator.new('String', ['strict', 'soft'])
      unless validator.valid?(truck_restriction_penalty)
        fail ArgumentError, 'invalid value for "truck_restriction_penalty", must be one of #{validator.allowable_values}.'
      end
      @truck_restriction_penalty = truck_restriction_penalty
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] truck_type Object to be assigned
    def truck_type=(truck_type)
      validator = EnumAttributeValidator.new('String', ['truck', 'tractorTruck'])
      unless validator.valid?(truck_type)
        fail ArgumentError, 'invalid value for "truck_type", must be one of #{validator.allowable_values}.'
      end
      @truck_type = truck_type
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] tunnel_category Object to be assigned
    def tunnel_category=(tunnel_category)
      validator = EnumAttributeValidator.new('String', ['B', 'C', 'D', 'E'])
      unless validator.valid?(tunnel_category)
        fail ArgumentError, 'invalid value for "tunnel_category", must be one of #{validator.allowable_values}.'
      end
      @tunnel_category = tunnel_category
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          height == o.height &&
          length == o.length &&
          limited_weight == o.limited_weight &&
          shipped_hazardous_goods == o.shipped_hazardous_goods &&
          trailers_count == o.trailers_count &&
          truck_restriction_penalty == o.truck_restriction_penalty &&
          truck_type == o.truck_type &&
          tunnel_category == o.tunnel_category &&
          weight_per_axle == o.weight_per_axle &&
          width == o.width
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [height, length, limited_weight, shipped_hazardous_goods, trailers_count, truck_restriction_penalty, truck_type, tunnel_category, weight_per_axle, width].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = SwaggerClient.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
